/* date_order - order filenames by date
**
** Copyright (C) 1995 by Jef Poskanzer <jef@netcom.com>.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <time.h>

extern time_t date_parse();

static void usage();
static time_t date_file();
static int file_compare();
static char* my_malloc();
static char* my_realloc();
static void out_of_mem();

static char* argv0;

struct file
    {
    char* name;
    time_t date;
    };

static int maxfiles, nfiles;
static struct file* files;

void
main( argc, argv )
    int argc;
    char* argv[];
    {
    int argn;
    int i;

    argv0 = argv[0];
    argn = 1;
    if ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' )
	usage();

    maxfiles = argc - argn;
    files = (struct file*) my_malloc( maxfiles * sizeof(struct file) );
    nfiles = 0;

    while ( argn < argc )
	{
	files[nfiles].name = (char*) my_malloc( strlen( argv[argn] ) + 1 );
	(void) strcpy( files[nfiles].name, argv[argn] );
	files[nfiles].date = date_file( argv[argn] );
	++nfiles;
	++argn;
	}

    qsort( files, nfiles, sizeof(struct file), file_compare );

    for ( i = 0; i < nfiles; ++i )
	{
	if ( i != 0 )
	    putchar( ' ' );
	fputs( files[i].name, stdout );
	}
    putchar( '\n' );

    exit( 0 );
    }

static void
usage()
    {
    (void) fprintf( stderr, "usage:  %s [file ...]\n", argv0 );
    exit( 1 );
    }

static time_t
date_file( name )
    char* name;
    {
    FILE* f;
    char line[30000];
    time_t t;

    f = fopen( name, "r" );
    if ( f == (FILE*) 0 )
	{
	perror( name );
	return (time_t) -1;
	}

    for (;;)
	{
	if ( fgets( line, sizeof(line), f ) == (char*) 0 )
	    break;
	if ( line[0] == '\n' )
	    break;
	if ( strncmp( line, "Date:", 5 ) == 0 )
	    {
	    (void) fclose( f );
	    t = date_parse( &line[5] );
	    if ( t == (time_t) -1 )
		(void) fprintf(
		    stderr, "%s: unparsable date in %s - %s",
		    argv0, name, &line[5] );
	    return t;
	    }
	}

    (void) fprintf( stderr, "%s: no Date: line found in %s\n", argv0, name );
    (void) fclose( f );
    return (time_t) -1;
    }

static int
file_compare( f1, f2 )
    struct file* f1;
    struct file* f2;
    {
    return f1->date - f2->date;
    }

static char*
my_malloc( size )
    unsigned size;
    {
    char* r;

    r = malloc( size );
    if ( r == (char*) 0 )
	out_of_mem();
    return r;
    }

static char*
my_realloc( p, size )
    char* p;
    unsigned size;
    {
    char* r;

    r = realloc( p, size );
    if ( r == (char*) 0 )
	out_of_mem();
    return r;
    }

static void
out_of_mem()
    {
    (void) fprintf( stderr, "%s: out of memory\n", argv0 );
    exit( 1 );
    }
