/* date_sort - sort input by date
**
** Copyright (C) 1995 by Jef Poskanzer <jef@netcom.com>.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <time.h>

extern time_t date_parse();

static void usage();
static void read_file();
static int line_compare();
static char* my_malloc();
static char* my_realloc();
static void out_of_mem();

static char* argv0;

struct line
    {
    char* text;
    time_t date;
    };

static int maxlines, nlines;
static struct line* lines;

void
main( argc, argv )
    int argc;
    char* argv[];
    {
    int argn;
    FILE* f;
    int i;

    argv0 = argv[0];
    argn = 1;
    if ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' )
	usage();

    maxlines = 500;		/* arbitrary initial value */
    lines = (struct line*) my_malloc( maxlines * sizeof(struct line) );
    nlines = 0;

    if ( argn == argc )
	read_file( stdin );
    else
	while ( argn < argc )
	    {
	    if ( strcmp( argv[argn], "-" ) == 0 )
		read_file( stdin );
	    else
		{
		f = fopen( argv[argn], "r" );
		if ( f == (FILE*) 0 )
		    {
		    perror( argv[argn] );
		    exit( 1 );
		    }
		read_file( f );
		(void) fclose( f );
		}
	    ++argn;
	    }

    qsort( lines, nlines, sizeof(struct line), line_compare );

    for ( i = 0; i < nlines; ++i )
	fputs( lines[i].text, stdout );

    exit( 0 );
    }

static void
usage()
    {
    (void) fprintf( stderr, "usage:  %s [file ...]\n", argv0 );
    exit( 1 );
    }

static void
read_file( f )
    FILE* f;
    {
    char aline[10000];

    /* Read until end of file. */
    while ( fgets( aline, sizeof(aline), f ) != (char*) 0 )
	{
	/* Make sure there's enough room in the lines array. */
	if ( nlines >= maxlines )
	    {
	    maxlines *= 2;
	    lines = (struct line*) my_realloc(
		(char*) lines, maxlines * sizeof(struct line) );
	    }
	lines[nlines].text = (char*) my_malloc( strlen( aline ) + 1 );
	(void) strcpy( lines[nlines].text, aline );
	lines[nlines].date = date_parse( aline );
	if ( lines[nlines].date == (time_t) -1 )
	    (void) fprintf( stderr, "%s: unparsable date - %s", argv0, aline );
	++nlines;
	}
    }

static int
line_compare( l1, l2 )
    struct line* l1;
    struct line* l2;
    {
    return l1->date - l2->date;
    }

static char*
my_malloc( size )
    unsigned size;
    {
    char* r;

    r = malloc( size );
    if ( r == (char*) 0 )
	out_of_mem();
    return r;
    }

static char*
my_realloc( p, size )
    char* p;
    unsigned size;
    {
    char* r;

    r = realloc( p, size );
    if ( r == (char*) 0 )
	out_of_mem();
    return r;
    }

static void
out_of_mem()
    {
    (void) fprintf( stderr, "%s: out of memory\n", argv0 );
    exit( 1 );
    }
