/* deltime - show difference between two dates
**
** Copyright (C) 1995 by Jef Poskanzer <jef@netcom.com>.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/

#include <stdio.h>
#include <time.h>

extern time_t date_parse();

static void usage();
static void unparsable();

static char* argv0;

#define SECSPERMINUTE 60
#define SECSPERHOUR (60 * SECSPERMINUTE)
#define SECSPERDAY (24 * SECSPERHOUR)

void
main( argc, argv )
    int argc;
    char* argv[];
    {
    time_t t1, t2;
    long delta;
    int days, hours, minutes, secs;

    argv0 = argv[0];

    if ( argc == 2 )
	{
	t1 = date_parse( argv[1] );
	if ( t1 == (time_t) -1 )
	    unparsable( argv[1] );
	t2 = time( (time_t*) 0 );
	}
    else if ( argc == 3 )
	{
	t1 = date_parse( argv[1] );
	if ( t1 == (time_t) -1 )
	    unparsable( argv[1] );
	t2 = date_parse( argv[2] );
	if ( t2 == (time_t) -1 )
	    unparsable( argv[2] );
	}
    else
	usage();

    delta = t2 - t1;
    if ( delta < 0 )
	{
	(void) printf( "-" );
	delta = -delta;
	}

    days = delta / SECSPERDAY;
    delta = delta - days * SECSPERDAY;
    hours = delta / SECSPERHOUR;
    delta = delta - hours * SECSPERHOUR;
    minutes = delta / SECSPERMINUTE;
    delta = delta - minutes * SECSPERMINUTE;
    secs = delta;

    printf( "%d %2d:%02d:%02d\n", days, hours, minutes, secs );

    exit( 0 );
    }

static void
usage()
    {
    (void) fprintf( stderr, "usage:  %s date1 [date2]\n", argv0 );
    exit( 1 );
    }

static void
unparsable( str )
    char* str;
    {
    (void) fprintf( stderr, "%s: unparsable date - %s\n", argv0, str );
    exit( 1 );
    }
