/* xmit_time - figure out transmission time of netnews articles
**
** Copyright (C) 1995 by Jef Poskanzer <jef@netcom.com>.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>

extern time_t date_parse();

static void usage();
static void do_file();

static char* argv0;

void
main( argc, argv )
    int argc;
    char* argv[];
    {
    int argn;

    argv0 = argv[0];
    argn = 1;
    if ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' )
	usage();

    while ( argn < argc )
	{
	do_file( argv[argn] );
	++argn;
	}

    exit( 0 );
    }

static void
usage()
    {
    (void) fprintf( stderr, "usage:  %s file ...\n", argv0 );
    exit( 1 );
    }

static void
do_file( name )
    char* name;
    {
    struct stat sb;
    time_t posted, arrived;
    FILE* f;
    char line[30000];

    f = fopen( name, "r" );
    if ( f == (FILE*) 0 )
	{
	perror( name );
	return;
	}

    if ( fstat( fileno( f ), &sb ) < 0 )
	{
	perror( name );
	(void) fclose( f );
	return;
	}
    arrived = sb.st_mtime;

    for (;;)
	{
	if ( fgets( line, sizeof(line), f ) == (char*) 0 )
	    break;
	if ( line[0] == '\n' )
	    break;
	if ( strncmp( line, "Date:", 5 ) == 0 )
	    {
	    (void) fclose( f );
	    posted = date_parse( &line[5] );
	    if ( posted == (time_t) -1 )
		(void) fprintf(
		    stderr, "%s: unparsable date in %s - %s",
		    argv0, name, &line[5] );
	    else
		(void) printf( "%d\n", arrived - posted );
	    return;
	    }
	}

    (void) fprintf( stderr, "%s: no Date: line found in %s\n", argv0, name );
    (void) fclose( f );
    }


